/*
 * Copyright 2009 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kantega.jexmec.spring;

import org.springframework.beans.factory.config.AbstractFactoryBean;
import org.springframework.beans.factory.config.BeanFactoryPostProcessor;
import org.springframework.beans.BeansException;
import org.springframework.context.*;
import org.springframework.context.event.ContextRefreshedEvent;
import org.kantega.jexmec.*;
import org.kantega.jexmec.simple.SimpleClassLoaderStrategy;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.net.URL;

/**
 * FactoryBean for easy creation and startup of a DefaultPluginManager in a Spring context.
 */
public class PluginManagerFactoryBean extends AbstractFactoryBean implements ApplicationContextAware, ApplicationListener {
    private ApplicationContext applicationContext;


    private Class pluginClass;

    private Class serviceClass;


    private ServiceLocator serviceLocator;

    private List<PluginLoader<Plugin>> pluginLoaders = Collections.emptyList();
    private List<ClassLoaderStrategy> classLoaderStrategies = Collections.emptyList();
    private boolean addSystemClassLoaderStrategy = true;


    public Class getObjectType() {
        return PluginManager.class;
    }

    public boolean isSingleton() {
        return true;
    }

    public Object createInstance() throws Exception {

        if (serviceLocator == null && serviceClass != null) {
            serviceLocator = new SpringServiceLocator(applicationContext, serviceClass);
        }


        final DefaultPluginManager manager = serviceLocator != null ? new DefaultPluginManager(pluginClass, serviceLocator) : new DefaultPluginManager(pluginClass);

        for (PluginLoader<Plugin> loader : pluginLoaders) {
            manager.addPluginLoader(loader);
        }
        if(addSystemClassLoaderStrategy) {
            manager.addClassLoaderStrategy(new SimpleClassLoaderStrategy(getClass().getClassLoader()));
        }
        for (ClassLoaderStrategy strategy : classLoaderStrategies) {
            manager.addClassLoaderStrategy(strategy);
        }

        return manager;
    }

    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }


    public void setPluginLoaders(List<PluginLoader<Plugin>> pluginLoaders) {
        this.pluginLoaders = pluginLoaders;
    }

    public void setServiceLocator(ServiceLocator serviceLocator) {
        this.serviceLocator = serviceLocator;
    }

    public void onApplicationEvent(ApplicationEvent event) {
        if (event instanceof ContextRefreshedEvent && event.getSource() == applicationContext) {
            try {
                DefaultPluginManager manager = (DefaultPluginManager) getObject();

                manager.start();
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    public void setServiceClass(Class serviceClass) {
        this.serviceClass = serviceClass;
    }

    public void setPluginClass(Class pluginClass) {
        this.pluginClass = pluginClass;
    }

    public void setClassLoaderStrategies(List<ClassLoaderStrategy> classLoaderStrategies) {
        this.classLoaderStrategies = classLoaderStrategies;
    }

    public void setAddSystemClassLoaderStrategy(boolean addSystemClassLoaderStrategy) {
        this.addSystemClassLoaderStrategy = addSystemClassLoaderStrategy;
    }
}
