package org.kantega.jexmec.spring;

/*
 * Copyright 2009 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.kantega.jexmec.*;
import org.springframework.beans.factory.ListableBeanFactory;

import java.util.*;

/**
 * {@link ServiceLocator} implementation where services are looked up in a Spring BeanFactory.
 */
public class SpringServiceLocator implements ServiceLocator {
    private Map<ServiceKey, Object> services;
    private final Set<ServiceKey> serviceKeys;
    private ListableBeanFactory beanFactory;

    /**
     * Creates a ServiceLocator that will look up service classes in the provided Spring Bean Factory.
     * @param beanFactory the bean factory to look in
     * @param servicesInterface the interface used to find service types
     */
    public <S extends Services> SpringServiceLocator(ListableBeanFactory beanFactory, Class<S> servicesInterface) {
        this(beanFactory, new DefaultServiceTypesProvider(servicesInterface));
    }

    /**
     * Creates a ServiceLocator that will look up service classes in the provided Spring Bean Factory.
     * @param beanFactory the bean factory to look in
     * @param serviceTypesProvider the {@link org.kantega.jexmec.ServiceTypesProvider} used to look up service classes
     */

    public SpringServiceLocator(ListableBeanFactory beanFactory, ServiceTypesProvider serviceTypesProvider) {
        this(beanFactory, serviceTypesProvider.getServiceKeys());
    }
    /**
     * Creates a ServiceLocator that will look up service classes in the provided Spring Bean Factory.
     * @param beanFactory the bean factory to look in
     * @param serviceKeys the service classes that will be looked up
     */
    public SpringServiceLocator(ListableBeanFactory beanFactory, Set<ServiceKey> serviceKeys) {
        this.beanFactory = beanFactory;
        this.serviceKeys = serviceKeys;
    }

    public Set<ServiceKey> getServiceKeys() {
        return serviceKeys;
    }

    public <T> T lookupService(Class<T> serviceClass) {
        return lookupService(serviceClass, null);
    }

    public <T> T lookupService(Class<T> serviceClass, String serviceName) {
        synchronized(this) {
            if(services == null) {
                services = Collections.synchronizedMap(new HashMap<ServiceKey, Object>());

                for (ServiceKey serviceKey : serviceKeys) {
                    @SuppressWarnings("unchecked")
                    Map<String, Object> beans = beanFactory.getBeansOfType(serviceKey.getServiceClass());
                    if (beans.size() == 0) {
                        throw new IllegalStateException("Application context " + beanFactory.toString() + " contains no bean of type " + serviceKey);
                    }
                    if (beans.size() > 1) {
                        if(serviceKey.getServiceName() == null) {
                            throw new IllegalStateException("Application context " + beanFactory.toString()
                                    + " contains multiple beans of type " + serviceKey.getServiceClass()
                                    +", service must be qualified with one of the following names: " + beans.keySet());
                        } else {
                            if(!beans.keySet().contains(serviceKey.getServiceName())) {
                                throw new IllegalStateException("Application context " + beanFactory.toString()
                                    + " contains no beans of type " + serviceKey.getServiceClass()
                                    +" named '" +serviceKey.getServiceName() + "'");
                            } else {
                                services.put(serviceKey, beans.get(serviceKey.getServiceName()));
                            }

                        }
                    } else {
                        services.put(serviceKey, beans.values().iterator().next());
                    }
                }
                services = Collections.unmodifiableMap(services);
            }
        }
        return serviceClass.cast(services.get(new ServiceKey(serviceClass, serviceName)));  
    }
}
